/**
 *******************************************************************************
 * @file    usbd_scsi.h
 * @brief   MSC(SCSI) Function Header for TOSHIBA 'TMPM366' Device Series
 * @version V0.201
 * @date    2014/05/20
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 *
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

#ifndef _USBD_SCSI_H_
#define _USBD_SCSI_H_

#include "usbd_hw.h"
#include "usbd_msc.h"

/* Media Status */
#define USBD_MSC_SCSI_MEDIASTATE_OK                         (0U)
#define USBD_MSC_SCSI_MEDIASTATE_ERROR                      (1U)
#define USBD_MSC_SCSI_MEDIASTATE_NO_MEDIA                   (2U)

/* Sense Key */
#define USBD_MSC_SCSI_SENSE_KEY_NO_SENSE                    (0x00U)
#define USBD_MSC_SCSI_SENSE_KEY_RECOVERED_ERROR             (0x01U)
#define USBD_MSC_SCSI_SENCE_KEY_NOT_READY                   (0x02U)
#define USBD_MSC_SCSI_SENSE_KEY_MEDIUM_ERROR                (0x03U)
#define USBD_MSC_SCSI_SENSE_KEY_HARDWARE_ERROR              (0x04U)
#define USBD_MSC_SCSI_SENSE_KEY_ILLEGAL_REQUEST             (0x05U)
#define USBD_MSC_SCSI_SENSE_KEY_UNIT_ATTENTION              (0x06U)
#define USBD_MSC_SCSI_SENSE_KEY_DATA_PROTECT                (0x07U)
#define USBD_MSC_SCSI_SENSE_KEY_BLANK_CHECK                 (0x08U)
#define USBD_MSC_SCSI_SENSE_KEY_VENDOR_SPECIFIC             (0x09U)
#define USBD_MSC_SCSI_SENSE_KEY_COPY_ABORTED                (0x0AU)
#define USBD_MSC_SCSI_SENSE_KEY_ABORTED_COMMAND             (0x0BU)
#define USBD_MSC_SCSI_SENSE_KEY_EQUAL                       (0x0CU)
#define USBD_MSC_SCSI_SENSE_KEY_VOLUME_OVERFLOW             (0x0DU)
#define USBD_MSC_SCSI_SENSE_KEY_MISCOMPARE                  (0x0EU)
#define USBD_MSC_SCSI_SENSE_KEY_RESERVED                    (0x0FU)

/* Mode Sense */
#define USBD_MSC_SCSI_PAGE_CONTROL_BIT                      (0xC0U)
#define USBD_MSC_SCSI_PAGE_CODE_BIT                         (0x3FU)

/* Page Control */
#define USBD_MSC_SCSI_MODESENSE_CURRENT                     (0U)
#define USBD_MSC_SCSI_MODESENSE_CHANGEABLE                  (1U)
#define USBD_MSC_SCSI_MODESENSE_DEFAULT                     (2U)
#define USBD_MSC_SCSI_MODESENSE_SAVED                       (3U)

/* Page Code */
#define USBD_MSC_SCSI_MODESENSE_VENDOR_FREE_FORMAT          (0x00U)
#define USBD_MSC_SCSI_MODESENSE_DEVICE_TYPE_SPEC_START      (0x01U)
#define USBD_MSC_SCSI_MODESENSE_DEVICE_TYPE_SPEC_END        (0x1FU)
#define USBD_MSC_SCSI_MODESENSE_VENDOR_PAGE_FORMAT_START    (0x20U)
#define USBD_MSC_SCSI_MODESENSE_VENDOR_PAGE_FORMAT_END      (0x3EU)
#define USBD_MSC_SCSI_MODESENSE_ALL_MODE_PAGE               (0x3FU)

/* Prevent/Allow Medium Removal */
#define USBD_MSC_SCSI_PREVENT_BIT                           (1U)

#define USBD_MSC_SCSI_MEDIA_REMOVAL_OK                      (0U)
#define USBD_MSC_SCSI_MEDIA_REMOVAL_NG                      (1U)

/* Start Stop Unit */
#define USBD_MSC_SCSI_START_BIT                             (1U)
#define USBD_MSC_SCSI_LOEJ_BIT                              (2U)

#define USBD_MSC_SCSI_AUTO_LOAD_EJECT_STOP                  (0U)
#define USBD_MSC_SCSI_AUTO_LOAD_EJECT_START                 (1U)

#define USBD_MSC_SCSI_MEDIA_ACCESS_STOP                     (0U)
#define USBD_MSC_SCSI_MEDIA_ACCESS_START                    (1U)

/* Read Capacity */
#define USBD_MSC_SCSI_PMI_BIT                               (1U)
#define USBD_MSC_SCSI_BLOCK_SIZE                            (512U)  /* Block size       */
#define USBD_MSC_SCSI_BLOCK_NUM                             (24U)   /* Number of blocks */

/* Verify */
#define USBD_MSC_SCSI_DPO_BIT                               (0x10U)
#define USBD_MSC_SCSI_BYTCHK_BIT                            (0x02U)
#define USBD_MSC_SCSI_RELADR_BIT                            (0x01U)

/* Operation Code */
#define USBD_MSC_SCSI_TEST_UNIT_READY                       (0x00U)
#define USBD_MSC_SCSI_REQUEST_SENCE                         (0x03U)
#define USBD_MSC_SCSI_INQUIRY                               (0x12U)
#define USBD_MSC_SCSI_MODE_SENSE                            (0x1AU)
#define USBD_MSC_SCSI_START_STOP_UNIT                       (0x1BU)
#define USBD_MSC_SCSI_PREVENT_ALLOW_MEDIUM_REMOVAL          (0x1EU)
#define USBD_MSC_SCSI_READ_CAPACITY                         (0x25U)
#define USBD_MSC_SCSI_READ                                  (0x28U)
#define USBD_MSC_SCSI_WRITE                                 (0x2AU)
#define USBD_MSC_SCSI_VERIFY                                (0x2FU)

/* Command Status */
#define USBD_MSC_SCSI_COMMAND_PASSED                        USBD_MSC_BOT_COMMAND_PASSED
#define USBD_MSC_SCSI_COMMAND_FAILED                        USBD_MSC_BOT_COMMAND_FAILED
#define USBD_MSC_SCSI_COMMAND_PHASE_ERROR                   USBD_MSC_BOT_COMMAND_PHASE_ERROR

/* STALL Request */
#define USBD_MSC_SCSI_REQ_STALL                             USBD_MSC_BOT_REQ_STALL
#define USBD_MSC_SCSI_NO_STALL                              USBD_MSC_BOT_NO_STALL

#define USBD_MSC_SCSI_NO_DATA                               (0U)

/* Reply Inquiry */
typedef struct {
    uint8_t       uc_Byte_0;
    uint8_t       uc_Byte_1;
    uint8_t       uc_Byte_2;
    uint8_t       uc_Byte_3;
    uint8_t       uc_Additional_Length;
    uint8_t       uc_Reserved1;
    uint8_t       uc_Reserved2;
    uint8_t       uc_Byte_7;
    uint8_t       auc_Vendor_ID[8U];
    uint8_t       auc_Product_ID[16U];
    uint8_t       auc_Product_Revision_Level[4U];
} USBD_MSC_SCSI_RET_INQUIRY_DATA_t;

/* Reply Sense Data */
typedef struct {
    uint8_t       uc_ErrorCode;
    uint8_t       uc_SegmentNumber;
    uint8_t       uc_Byte_2;
    uint8_t       auc_Infomation[4U];
    uint8_t       uc_AdditionalSenseLength;
    uint8_t       auc_CommandSpecificInfo[4U];
    uint8_t       uc_AdditionalSenseCode;
    uint8_t       uc_AdditionalSenseCodeQualifier;
    uint8_t       uc_FRU_Code;
    uint8_t       auc_SenseKey_Specific[3U];
} USBD_MSC_SCSI_RET_SENSE_DATA_t;

/* Reply Mode Sense Data (Header) */
typedef struct {
    uint8_t       uc_ModeDataLen;
    uint8_t       uc_MediumType;
    uint8_t       uc_Dev_Spec_Param;
    uint8_t       uc_Block_Desc_Len;
} USBD_MSC_SCSI_RET_MODE_SENSE_HEADER_t;

/* Reply Mode Sense Data */
typedef struct {
    USBD_MSC_SCSI_RET_MODE_SENSE_HEADER_t           tHeader;
} USBD_MSC_SCSI_RET_MODE_SENSE_DATA_t;

/* Reply Read Capacity Data */
typedef struct {
    uint8_t       uc_LogicalBlockAddr[4U];
    uint8_t       uc_BlockLength[4U];
} USBD_MSC_SCSI_RET_READ_CAPACITY_DATA_t;

/* Typedef for MSC(SCSI) Callback Functions */
typedef void (*SCSI_CB_Media)(uint32_t, uint32_t, uint16_t);
typedef void (*SCSI_CB_TrnsData)(uint32_t, uint8_t*);

/* Callback Information */
typedef struct {
    MSC_CB_ConnStat                 pfnConnStat;
    MSC_CB_MassStorageReset         pfnMassStorageReset;
    MSC_CB_GetMaxLun                pfnGetMaxLun;
    SCSI_CB_Media                   pfnMediaRead;
    SCSI_CB_Media                   pfnMediaWrite;
    SCSI_CB_TrnsData                pfnSendData;
    SCSI_CB_TrnsData                pfnRecvData;
} SCSI_CB_t;

/* MSC(SCSI) Initialize */
void USBD_MSC_SCSI_Init(SCSI_CB_t*);

/* Other Prototypes */
bool USBD_MSC_SCSI_MediaReadData(uint32_t, uint8_t*, void*);
bool USBD_MSC_SCSI_MediaWriteData(uint32_t, uint8_t*, void*);
void USBD_MSC_SCSI_MediaReadResult(uint8_t, uint32_t, bool);
void USBD_MSC_SCSI_MediaWriteResult(uint8_t, uint32_t, bool);
void USBD_MSC_SCSI_Media_ConnStat(uint8_t);
void USBD_MSC_SCSI_SetSenseKey(uint8_t);

#endif  /* _USBD_SCSI_H_ */
